@@ -58,9 +58,9 @@
 #include <Qt3DRender/private/effect_p.h>
 #include <Qt3DRender/private/entity_p.h>
 #include <Qt3DRender/private/layer_p.h>
+#include <Qt3DRender/private/levelofdetail_p.h>
 #include <Qt3DRender/private/material_p.h>
 #include <Qt3DRender/private/shader_p.h>
-#include <Qt3DRender/private/technique_p.h>
 #include <Qt3DRender/private/texture_p.h>
 #include <Qt3DRender/private/transform_p.h>
 #include <Qt3DRender/private/rendertarget_p.h>
@@ -77,6 +77,7 @@
 #include <Qt3DRender/private/boundingvolumedebug_p.h>
 #include <Qt3DRender/private/openglvertexarrayobject_p.h>
 #include <Qt3DRender/private/light_p.h>
+#include <Qt3DRender/private/environmentlight_p.h>
 #include <Qt3DRender/private/computecommand_p.h>
 
 QT_BEGIN_NAMESPACE
@@ -90,7 +91,7 @@ class AttachmentManager : public Qt3DCore::QResourceManager<
         Qt3DCore::QNodeId,
         16,
         Qt3DCore::ArrayAllocatingPolicy,
-        Qt3DCore::ObjectLevelLockingPolicy>
+        Qt3DCore::NonLockingPolicy>
 {
 public:
     AttachmentManager() {}
@@ -100,7 +101,8 @@ class CameraManager : public Qt3DCore::QResourceManager<
         CameraLens,
         Qt3DCore::QNodeId,
         8,
-        Qt3DCore::ArrayAllocatingPolicy>
+        Qt3DCore::ArrayAllocatingPolicy,
+        Qt3DCore::NonLockingPolicy>
 {
 public:
     CameraManager() {}
@@ -111,7 +113,7 @@ class FilterKeyManager : public Qt3DCore::QResourceManager<
         Qt3DCore::QNodeId,
         16,
         Qt3DCore::ArrayAllocatingPolicy,
-        Qt3DCore::ObjectLevelLockingPolicy>
+        Qt3DCore::NonLockingPolicy>
 
 {
 public:
@@ -123,7 +125,7 @@ class EffectManager : public Qt3DCore::QResourceManager<
         Qt3DCore::QNodeId,
         16,
         Qt3DCore::ArrayAllocatingPolicy,
-        Qt3DCore::ObjectLevelLockingPolicy>
+        Qt3DCore::NonLockingPolicy>
 {
 public:
     EffectManager() {}
@@ -132,7 +134,9 @@ public:
 class Q_AUTOTEST_EXPORT EntityManager : public Qt3DCore::QResourceManager<
         Entity,
         Qt3DCore::QNodeId,
-        16>
+        16,
+        Qt3DCore::ArrayAllocatingPolicy,
+        Qt3DCore::NonLockingPolicy>
 {
 public:
     EntityManager() {}
@@ -168,24 +172,40 @@ class LayerManager : public Qt3DCore::QResourceManager<
         Qt3DCore::QNodeId,
         16,
         Qt3DCore::ArrayAllocatingPolicy,
-        Qt3DCore::ObjectLevelLockingPolicy>
+        Qt3DCore::NonLockingPolicy>
 {
 public:
     LayerManager() {}
 };
 
+class LevelOfDetailManager : public Qt3DCore::QResourceManager<
+        LevelOfDetail,
+        Qt3DCore::QNodeId,
+        16,
+        Qt3DCore::ArrayAllocatingPolicy,
+        Qt3DCore::NonLockingPolicy>
+{
+public:
+    LevelOfDetailManager() {}
+};
+
 class MaterialManager : public Qt3DCore::QResourceManager<
         Material,
         Qt3DCore::QNodeId,
         16,
         Qt3DCore::ArrayAllocatingPolicy,
-        Qt3DCore::ObjectLevelLockingPolicy>
+        Qt3DCore::NonLockingPolicy>
 {
 public:
     MaterialManager() {}
 };
 
-class MatrixManager : public Qt3DCore::QResourceManager<QMatrix4x4, Qt3DCore::QNodeId, 16>
+class MatrixManager : public Qt3DCore::QResourceManager<
+        QMatrix4x4,
+        Qt3DCore::QNodeId,
+        16,
+        Qt3DCore::ArrayAllocatingPolicy,
+        Qt3DCore::NonLockingPolicy>
 {
 public:
     MatrixManager() {}
@@ -196,32 +216,36 @@ class ShaderManager : public Qt3DCore::QResourceManager<
         Qt3DCore::QNodeId,
         16,
         Qt3DCore::ArrayAllocatingPolicy,
-        Qt3DCore::ObjectLevelLockingPolicy>
+        Qt3DCore::NonLockingPolicy>
 {
 public:
     ShaderManager() {}
 };
 
-class TechniqueManager : public Qt3DCore::QResourceManager<
-        Technique,
-        Qt3DCore::QNodeId,
-        16,
-        Qt3DCore::ArrayAllocatingPolicy,
-        Qt3DCore::ObjectLevelLockingPolicy>
-{
-public:
-    TechniqueManager() {}
-};
-
 class TextureManager : public Qt3DCore::QResourceManager<
         Texture,
         Qt3DCore::QNodeId,
         16,
         Qt3DCore::ArrayAllocatingPolicy,
-        Qt3DCore::ObjectLevelLockingPolicy>
+        Qt3DCore::NonLockingPolicy>
 {
 public:
     TextureManager() {}
+
+    // Called in AspectThread by Texture node functor destroy
+    void addTextureIdToCleanup(Qt3DCore::QNodeId id)
+    {
+        m_textureIdsToCleanup.push_back(id);
+    }
+
+    // Called by RenderThread in updateGLResources (locked)
+    QVector<Qt3DCore::QNodeId> takeTexturesIdsToCleanup()
+    {
+        return std::move(m_textureIdsToCleanup);
+    }
+
+private:
+    QVector<Qt3DCore::QNodeId> m_textureIdsToCleanup;
 };
 
 class TransformManager : public Qt3DCore::QResourceManager<
@@ -229,7 +253,7 @@ class TransformManager : public Qt3DCore::QResourceManager<
         Qt3DCore::QNodeId,
         16,
         Qt3DCore::ArrayAllocatingPolicy,
-        Qt3DCore::ObjectLevelLockingPolicy>
+        Qt3DCore::NonLockingPolicy>
 {
 public:
     TransformManager() {}
@@ -237,8 +261,10 @@ public:
 
 class VAOManager : public Qt3DCore::QResourceManager<
         OpenGLVertexArrayObject,
-        QPair<HGeometry, HShader>,
-        16>
+        VAOIdentifier,
+        16,
+        Qt3DCore::ArrayAllocatingPolicy,
+        Qt3DCore::NonLockingPolicy>
 {
 public:
     VAOManager() {}
@@ -249,7 +275,7 @@ class RenderTargetManager : public Qt3DCore::QResourceManager<
         Qt3DCore::QNodeId,
         8,
         Qt3DCore::ArrayAllocatingPolicy,
-        Qt3DCore::ObjectLevelLockingPolicy>
+        Qt3DCore::NonLockingPolicy>
 {
 public:
     RenderTargetManager() {}
@@ -260,7 +286,7 @@ class RenderPassManager : public Qt3DCore::QResourceManager<
         Qt3DCore::QNodeId,
         16,
         Qt3DCore::ArrayAllocatingPolicy,
-        Qt3DCore::ObjectLevelLockingPolicy>
+        Qt3DCore::NonLockingPolicy>
 {
 public:
     RenderPassManager() {}
@@ -272,7 +298,7 @@ class ParameterManager : public Qt3DCore::QResourceManager<
         Qt3DCore::QNodeId,
         16,
         Qt3DCore::ArrayAllocatingPolicy,
-        Qt3DCore::ObjectLevelLockingPolicy>
+        Qt3DCore::NonLockingPolicy>
 {
 public:
     ParameterManager() {}
@@ -283,7 +309,7 @@ class ShaderDataManager : public Qt3DCore::QResourceManager<
         Qt3DCore::QNodeId,
         16,
         Qt3DCore::ArrayAllocatingPolicy,
-        Qt3DCore::ObjectLevelLockingPolicy>
+        Qt3DCore::NonLockingPolicy>
 {
 public:
     ShaderDataManager() {}
@@ -294,7 +320,7 @@ class GLBufferManager : public Qt3DCore::QResourceManager<
         Qt3DCore::QNodeId,
         16,
         Qt3DCore::ArrayAllocatingPolicy,
-        Qt3DCore::ObjectLevelLockingPolicy>
+        Qt3DCore::NonLockingPolicy>
 {
 };
 
@@ -303,7 +329,7 @@ class TextureImageManager : public Qt3DCore::QResourceManager<
         Qt3DCore::QNodeId,
         16,
         Qt3DCore::ArrayAllocatingPolicy,
-        Qt3DCore::ObjectLevelLockingPolicy>
+        Qt3DCore::NonLockingPolicy>
 {
 };
 
@@ -312,7 +338,7 @@ class AttributeManager : public Qt3DCore::QResourceManager<
         Qt3DCore::QNodeId,
         20,
         Qt3DCore::ArrayAllocatingPolicy,
-        Qt3DCore::ObjectLevelLockingPolicy>
+        Qt3DCore::NonLockingPolicy>
 {
 };
 
@@ -321,7 +347,7 @@ class GeometryManager : public Qt3DCore::QResourceManager<
         Qt3DCore::QNodeId,
         16,
         Qt3DCore::ArrayAllocatingPolicy,
-        Qt3DCore::ObjectLevelLockingPolicy>
+        Qt3DCore::NonLockingPolicy>
 {
 };
 
@@ -330,7 +356,7 @@ class ObjectPickerManager : public Qt3DCore::QResourceManager<
         Qt3DCore::QNodeId,
         16,
         Qt3DCore::ArrayAllocatingPolicy,
-        Qt3DCore::ObjectLevelLockingPolicy>
+        Qt3DCore::NonLockingPolicy>
 {
 };
 
@@ -350,18 +376,29 @@ class LightManager : public Qt3DCore::QResourceManager<
         Qt3DCore::QNodeId,
         16,
         Qt3DCore::ArrayAllocatingPolicy,
-        Qt3DCore::ObjectLevelLockingPolicy>
+        Qt3DCore::NonLockingPolicy>
 {
 public:
     LightManager() {}
 };
 
+class EnvironmentLightManager : public Qt3DCore::QResourceManager<
+        EnvironmentLight,
+        Qt3DCore::QNodeId,
+        16,
+        Qt3DCore::ArrayAllocatingPolicy,
+        Qt3DCore::NonLockingPolicy>
+{
+public:
+    EnvironmentLightManager() {}
+};
+
 class ComputeCommandManager : public Qt3DCore::QResourceManager<
         ComputeCommand,
         Qt3DCore::QNodeId,
         16,
         Qt3DCore::ArrayAllocatingPolicy,
-        Qt3DCore::ObjectLevelLockingPolicy>
+        Qt3DCore::NonLockingPolicy>
 {
 public:
     ComputeCommandManager() {}
@@ -372,7 +409,7 @@ class RenderStateManager : public Qt3DCore::QResourceManager<
         Qt3DCore::QNodeId,
         16,
         Qt3DCore::ArrayAllocatingPolicy,
-        Qt3DCore::ObjectLevelLockingPolicy>
+        Qt3DCore::NonLockingPolicy>
 {
 };
 
@@ -387,7 +424,6 @@ Q_DECLARE_RESOURCE_INFO(Qt3DRender::Render::Layer, Q_REQUIRES_CLEANUP)
 Q_DECLARE_RESOURCE_INFO(Qt3DRender::Render::Material, Q_REQUIRES_CLEANUP)
 Q_DECLARE_RESOURCE_INFO(Qt3DRender::Render::Shader, Q_REQUIRES_CLEANUP)
 Q_DECLARE_RESOURCE_INFO(Qt3DRender::Render::RenderTarget, Q_REQUIRES_CLEANUP)
-Q_DECLARE_RESOURCE_INFO(Qt3DRender::Render::Technique, Q_REQUIRES_CLEANUP)
 Q_DECLARE_RESOURCE_INFO(Qt3DRender::Render::Texture, Q_REQUIRES_CLEANUP)
 Q_DECLARE_RESOURCE_INFO(Qt3DRender::Render::RenderPass, Q_REQUIRES_CLEANUP)
 Q_DECLARE_RESOURCE_INFO(Qt3DRender::Render::TextureImage, Q_REQUIRES_CLEANUP)
@@ -396,6 +432,9 @@ Q_DECLARE_RESOURCE_INFO(Qt3DRender::Render::Geometry, Q_REQUIRES_CLEANUP)
 Q_DECLARE_RESOURCE_INFO(Qt3DRender::Render::ObjectPicker, Q_REQUIRES_CLEANUP)
 Q_DECLARE_RESOURCE_INFO(Qt3DRender::Render::BoundingVolumeDebug, Q_REQUIRES_CLEANUP)
 Q_DECLARE_RESOURCE_INFO(Qt3DRender::Render::ComputeCommand, Q_REQUIRES_CLEANUP)
+Q_DECLARE_RESOURCE_INFO(Qt3DRender::Render::Parameter, Q_REQUIRES_CLEANUP)
+Q_DECLARE_RESOURCE_INFO(Qt3DRender::Render::Transform, Q_REQUIRES_CLEANUP)
+Q_DECLARE_RESOURCE_INFO(Qt3DRender::Render::OpenGLVertexArrayObject, Q_REQUIRES_CLEANUP)
 
 QT_END_NAMESPACE
 
