@@ -149,6 +149,11 @@ void GraphicsHelperGL3_3::drawElements(GLenum primitiveType,
                                       baseVertex);
 }
 
+void GraphicsHelperGL3_3::drawElementsIndirect(GLenum, GLenum, void *)
+{
+    qWarning() << "Indirect Drawing is not supported with OpenGL 3";
+}
+
 void GraphicsHelperGL3_3::drawArrays(GLenum primitiveType,
                                       GLint first,
                                       GLsizei count)
@@ -158,6 +163,11 @@ void GraphicsHelperGL3_3::drawArrays(GLenum primitiveType,
                           count);
 }
 
+void GraphicsHelperGL3_3::drawArraysIndirect(GLenum , void *)
+{
+    qWarning() << "Indirect Drawing is not supported with OpenGL 3";
+}
+
 void GraphicsHelperGL3_3::setVerticesPerPatch(GLint verticesPerPatch)
 {
 #if defined(QT_OPENGL_4)
@@ -784,6 +794,12 @@ GLint GraphicsHelperGL3_3::maxClipPlaneCount()
     return max;
 }
 
+void GraphicsHelperGL3_3::memoryBarrier(QMemoryBarrier::Operations barriers)
+{
+    Q_UNUSED(barriers);
+    qWarning() << "memory barrier is not supported by OpenGL 3.3 (since 4.3)";
+}
+
 void GraphicsHelperGL3_3::enablePrimitiveRestart(int primitiveRestartIndex)
 {
     m_funcs->glPrimitiveRestartIndex(primitiveRestartIndex);
@@ -863,6 +879,16 @@ void GraphicsHelperGL3_3::dispatchCompute(GLuint wx, GLuint wy, GLuint wz)
     qWarning() << "Compute Shaders are not supported by OpenGL 3.3 (since OpenGL 4.3)";
 }
 
+char *GraphicsHelperGL3_3::mapBuffer(GLenum target)
+{
+    return static_cast<char*>(m_funcs->glMapBuffer(target, GL_READ_WRITE));
+}
+
+GLboolean GraphicsHelperGL3_3::unmapBuffer(GLenum target)
+{
+    return m_funcs->glUnmapBuffer(target);
+}
+
 void GraphicsHelperGL3_3::glUniform1fv(GLint location, GLsizei count, const GLfloat *values)
 {
     m_funcs->glUniform1fv(location, count, values);
