@@ -390,7 +390,7 @@ const struct DX10Format
 { DXGI_FORMAT_R8_UNORM,                             { QOpenGLTexture::Red,              QOpenGLTexture::R8_UNorm,       QOpenGLTexture::UInt8,           1, false } },
 { DXGI_FORMAT_R8G8_UNORM,                           { QOpenGLTexture::RG,               QOpenGLTexture::RG8_UNorm,      QOpenGLTexture::UInt8,           2, false } },
 { DXGI_FORMAT_R16_UNORM,                            { QOpenGLTexture::Red,              QOpenGLTexture::R16_UNorm,      QOpenGLTexture::UInt16,          2, false } },
-{ DXGI_FORMAT_R16_UNORM,                            { QOpenGLTexture::RG,               QOpenGLTexture::RG16_UNorm,     QOpenGLTexture::UInt16,          4, false } },
+{ DXGI_FORMAT_R16G16_UNORM,                         { QOpenGLTexture::RG,               QOpenGLTexture::RG16_UNorm,     QOpenGLTexture::UInt16,          4, false } },
 
 // depth formats
 { DXGI_FORMAT_D16_UNORM,                            { QOpenGLTexture::Depth,            QOpenGLTexture::D16,            QOpenGLTexture::NoPixelType,     2, false } },
@@ -418,16 +418,6 @@ const struct DX10Format
 { DXGI_FORMAT_BC7_UNORM_SRGB,                       { QOpenGLTexture::NoSourceFormat,   QOpenGLTexture::SRGB_BP_UNorm,  QOpenGLTexture::NoPixelType,    16, true } },
 };
 
-int bitCount(quint32 n)
-{
-    int r = 0;
-
-    for (; n; n >>= 1)
-        r += (n & 1);
-
-    return r;
-}
-
 enum CompressedFormatExtension {
     None = 0,
     DDS,
@@ -506,17 +496,17 @@ QTextureImageDataPtr setDdsFile(const QString &source)
             layers = qFromLittleEndian(dx10Header.arraySize);
             DXGIFormat format = static_cast<DXGIFormat>(qFromLittleEndian(dx10Header.format));
 
-            for (auto i = 0U; i < sizeof(dx10Formats) / sizeof(*dx10Formats); ++i) {
-                if (dx10Formats[i].dxgiFormat == format) {
-                    formatInfo = &dx10Formats[i].formatInfo;
+            for (const auto &info : dx10Formats) {
+                if (info.dxgiFormat == format) {
+                    formatInfo = &info.formatInfo;
                     break;
                 }
             }
         } else {
             // compressed, FourCC texture
-            for (auto i = 0U; i < sizeof(fourCCFormats) / sizeof(*fourCCFormats); ++i) {
-                if (fourCCFormats[i].fourCC == fourCC) {
-                    formatInfo = &fourCCFormats[i].formatInfo;
+            for (const auto &info : fourCCFormats) {
+                if (info.fourCC == fourCC) {
+                    formatInfo = &info.formatInfo;
                     break;
                 }
             }
@@ -529,13 +519,11 @@ QTextureImageDataPtr setDdsFile(const QString &source)
         const quint32 blueMask      = qFromLittleEndian(header.pixelFormat.blueMask);
         const quint32 alphaMask     = qFromLittleEndian(header.pixelFormat.alphaMask);
 
-        for (unsigned i = 0; i < sizeof rgbaFormats/sizeof *rgbaFormats; i++) {
-            const RGBAFormat *info = &rgbaFormats[i];
-
-            if (info->formatInfo.components * 8u == rgbBitCount &&
-                    info->redMask == redMask && info->greenMask == greenMask &&
-                    info->blueMask == blueMask && info->alphaMask == alphaMask) {
-                formatInfo = &info->formatInfo;
+        for (const auto &info : rgbaFormats) {
+            if (info.formatInfo.components * 8u == rgbBitCount &&
+                    info.redMask == redMask && info.greenMask == greenMask &&
+                    info.blueMask == blueMask && info.alphaMask == alphaMask) {
+                formatInfo = &info.formatInfo;
                 break;
             }
         }
@@ -565,7 +553,7 @@ QTextureImageDataPtr setDdsFile(const QString &source)
     } else if ((caps2Flags & CubemapFlag) == CubemapFlag) {
         target = layers > 1 ? QOpenGLTexture::TargetCubeMapArray : QOpenGLTexture::TargetCubeMap;
         depth = 1;
-        faces = bitCount(caps2Flags & AllCubemapFaceFlags);
+        faces = qPopulationCount(caps2Flags & AllCubemapFaceFlags);
     } else {
         target = layers > 1 ? QOpenGLTexture::Target2DArray : QOpenGLTexture::Target2D;
         depth = 1;
@@ -592,11 +580,17 @@ QTextureImageDataPtr setDdsFile(const QString &source)
     layerSize = faces * tmpSize;
 
     // data
-    const QByteArray data = f.readAll();
-    if (data.size() != layers * layerSize) {
-        qWarning() << "Unexpected data size (got " << data.size() << ", expecting" << layers * layerSize << ")";
+    const int dataSize = layers * layerSize;
+
+    const QByteArray data = f.read(dataSize);
+    if (data.size() < dataSize) {
+        qWarning() << "Unexpected end of data in" << source;
         return imageData;
     }
+
+    if (!f.atEnd())
+        qWarning() << "Unrecognized data in" << source;
+
     imageData = QTextureImageDataPtr::create();
     imageData->setData(data,blockSize, isCompressed);
 
@@ -623,10 +617,14 @@ QTextureImageDataPtr setDdsFile(const QString &source)
 
 } // anonynous
 
-QTextureImageDataPtr TextureLoadingHelper::loadTextureData(const QUrl &url, bool allow3D)
+QTextureImageDataPtr TextureLoadingHelper::loadTextureData(const QUrl &url, bool allow3D, bool mirrored)
 {
     QTextureImageDataPtr textureData;
-    if (url.isLocalFile() || url.scheme() == QLatin1String("qrc")) {
+    if (url.isLocalFile() || url.scheme() == QLatin1String("qrc")
+#ifdef Q_OS_ANDROID
+            || url.scheme() == QLatin1String("assets")
+#endif
+            ) {
         const QString source = Qt3DRender::QUrlHelper::urlToLocalFileOrQrc(url);
         const CompressedFormatExtension formatExtension = texturedCompressedFormat(source);
         switch (formatExtension) {
@@ -640,7 +638,7 @@ QTextureImageDataPtr TextureLoadingHelper::loadTextureData(const QUrl &url, bool
             QImage img;
             if (img.load(source)) {
                 textureData = QTextureImageDataPtr::create();
-                textureData->setImage(img.mirrored());
+                textureData->setImage(mirrored ? img.mirrored() : img);
             }
             break;
         }
@@ -656,7 +654,11 @@ QTextureDataPtr QTextureFromSourceGenerator::operator ()()
     QTextureDataPtr generatedData = QTextureDataPtr::create();
     m_status = QAbstractTexture::Loading;
 
-    const QTextureImageDataPtr textureData = TextureLoadingHelper::loadTextureData(m_url, true);
+    const QTextureImageDataPtr textureData = TextureLoadingHelper::loadTextureData(m_url, true, m_mirrored);
+
+    // Update any properties explicitly set by the user
+    if (m_format != QAbstractTexture::NoFormat && m_format != QAbstractTexture::Automatic)
+        textureData->setFormat(static_cast<QOpenGLTexture::TextureFormat>(m_format));
 
     if (textureData && textureData->data().length() > 0) {
         generatedData->setTarget(static_cast<QAbstractTexture::Target>(textureData->target()));
@@ -666,7 +668,6 @@ QTextureDataPtr QTextureFromSourceGenerator::operator ()()
         generatedData->setDepth(textureData->depth());
         generatedData->setLayers(textureData->layers());
         generatedData->addImageData(textureData);
-        // TO DO: Check that we aren't forgetting to set something here
         m_status = QAbstractTexture::Ready;
     } else {
         m_status = QAbstractTexture::Error;
@@ -674,11 +675,6 @@ QTextureDataPtr QTextureFromSourceGenerator::operator ()()
     return generatedData;
 }
 
-QTextureLoaderPrivate::QTextureLoaderPrivate()
-    : QAbstractTexturePrivate()
-{
-}
-
 /*!
     \class Qt3DRender::QTexture1D
     \inmodule Qt3DRender
@@ -899,13 +895,46 @@ QTextureBuffer::~QTextureBuffer()
 {
 }
 
+QTextureLoaderPrivate::QTextureLoaderPrivate()
+    : QAbstractTexturePrivate()
+    , m_mirrored(true)
+{
+}
+
+void QTextureLoaderPrivate::updateGenerator()
+{
+    Q_Q(QTextureLoader);
+    setDataFunctor(QTextureFromSourceGeneratorPtr::create(q));
+}
+
 /*!
  * Constructs a new Qt3DRender::QTextureLoader instance with \a parent as parent.
+ *
+ * Note that by default, if not contradicted by the file metadata, the loaded texture
+ * will have the following properties set:
+ *  - wrapMode set to Repeat
+ *  - minificationFilter set to LinearMipMapLinear
+ *  - magnificationFilter set to Linear
+ *  - generateMipMaps set to true
+ *  - maximumAnisotropy set to 16.0f
+ *  - target set to TargetAutomatic
  */
 QTextureLoader::QTextureLoader(QNode *parent)
     : QAbstractTexture(*new QTextureLoaderPrivate, parent)
 {
+    d_func()->m_wrapMode.setX(QTextureWrapMode::Repeat);
+    d_func()->m_wrapMode.setY(QTextureWrapMode::Repeat);
+    d_func()->m_minFilter = LinearMipMapLinear;
+    d_func()->m_magFilter = Linear;
+    d_func()->m_autoMipMap = true;
+    d_func()->m_maximumAnisotropy = 16.0f;
     d_func()->m_target = TargetAutomatic;
+
+    // Regenerate the texture functor when properties we support overriding
+    // from QAbstractTexture get changed.
+    Q_D(QTextureLoader);
+    auto regenerate = [=] () { d->updateGenerator(); };
+    connect(this, &QAbstractTexture::formatChanged, regenerate);
 }
 
 /*! \internal */
@@ -922,6 +951,12 @@ QUrl QTextureLoader::source() const
     return d->m_source;
 }
 
+bool QTextureLoader::isMirrored() const
+{
+    Q_D(const QTextureLoader);
+    return d->m_mirrored;
+}
+
 /*!
  * Sets the texture loader source to \a source.
  * \param source
@@ -931,11 +966,89 @@ void QTextureLoader::setSource(const QUrl& source)
     Q_D(QTextureLoader);
     if (source != d->m_source) {
         d->m_source = source;
-        d->m_dataFunctor.reset(new QTextureFromSourceGenerator(source));
+        d->updateGenerator();
+        const bool blocked = blockNotifications(true);
         emit sourceChanged(source);
+        blockNotifications(blocked);
     }
 }
 
+/*!
+  \property Qt3DRender::QTextureLoader::mirrored
+
+  This property specifies whether the texture should be mirrored when loaded. This
+  is a convenience to avoid having to manipulate images to match the origin of
+  the texture coordinates used by the rendering API. By default this property
+  is set to true. This has no effect when using compressed texture formats.
+
+  \note OpenGL specifies the origin of texture coordinates from the lower left
+  hand corner whereas DirectX uses the the upper left hand corner.
+
+  \note When using cube map texture you'll probably want mirroring disabled as
+  the cube map sampler takes a direction rather than regular texture
+  coordinates.
+*/
+
+/*!
+  \qmlproperty bool Qt3DRender::QTextureLoader::mirrored
+
+  This property specifies whether the texture should be mirrored when loaded. This
+  is a convenience to avoid having to manipulate images to match the origin of
+  the texture coordinates used by the rendering API. By default this property
+  is set to true. This has no effect when using compressed texture formats.
+
+  \note OpenGL specifies the origin of texture coordinates from the lower left
+  hand corner whereas DirectX uses the the upper left hand corner.
+
+  \note When using cube map texture you'll probably want mirroring disabled as
+  the cube map sampler takes a direction rather than regular texture
+  coordinates.
+*/
+
+/*!
+    Sets mirroring to \a mirrored.
+    \note This internally triggers a call to update the data generator.
+ */
+void QTextureLoader::setMirrored(bool mirrored)
+{
+    Q_D(QTextureLoader);
+    if (mirrored != d->m_mirrored) {
+        d->m_mirrored = mirrored;
+        d->updateGenerator();
+        const bool blocked = blockNotifications(true);
+        emit mirroredChanged(mirrored);
+        blockNotifications(blocked);
+    }
+}
+
+/*!
+ * Constructs a new QTextureFromSourceGenerator::QTextureFromSourceGenerator
+ * instance with properties passed in via \a textureLoader
+ * \param url
+ */
+QTextureFromSourceGenerator::QTextureFromSourceGenerator(QTextureLoader *textureLoader)
+    : QTextureGenerator()
+    , m_status(QAbstractTexture::None)
+    , m_url()
+    , m_mirrored()
+    , m_format(QAbstractTexture::RGBA8_UNorm)
+{
+    Q_ASSERT(textureLoader);
+
+    // We always get QTextureLoader's "own" additional properties
+    m_url = textureLoader->source();
+    m_mirrored = textureLoader->isMirrored();
+
+    // For the properties on the base QAbstractTexture we only apply
+    // those that have been explicitly set and which we support here.
+    // For more control, the user can themselves use a QTexture2D and
+    // create the texture images themselves, or even better, go create
+    // proper texture files themselves (dds/ktx etc). This is purely a
+    // convenience for some common use cases and will always be less
+    // ideal than using compressed textures and generating mips offline.
+    m_format = textureLoader->format();
+}
+
 /*!
  * Takes in a TextureGenerator via \a other and
  * \return whether generators have the same source.
@@ -943,19 +1056,20 @@ void QTextureLoader::setSource(const QUrl& source)
 bool QTextureFromSourceGenerator::operator ==(const QTextureGenerator &other) const
 {
     const QTextureFromSourceGenerator *otherFunctor = functor_cast<QTextureFromSourceGenerator>(&other);
-    return (otherFunctor != Q_NULLPTR && otherFunctor->m_url == m_url);
+    return (otherFunctor != nullptr &&
+            otherFunctor->m_url == m_url &&
+            otherFunctor->m_mirrored == m_mirrored &&
+            otherFunctor->m_format == m_format);
 }
 
-/*!
- * Constructs a new QTextureFromSourceGenerator::QTextureFromSourceGenerator
- * instance with \a url.
- * \param url
- */
-QTextureFromSourceGenerator::QTextureFromSourceGenerator(const QUrl &url)
-    : QTextureGenerator()
-    , m_url(url)
-    , m_status(QAbstractTexture::None)
+QUrl QTextureFromSourceGenerator::url() const
 {
+    return m_url;
+}
+
+bool QTextureFromSourceGenerator::isMirrored() const
+{
+    return m_mirrored;
 }
 
 } // namespace Qt3DRender
